@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2018-2020 EVA
+# Copyright 2018-2022 EVA
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,25 +13,42 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 from eva.expression.abstract_expression import AbstractExpression
-from eva.optimizer.operators import (LogicalCreateMaterializedView, LogicalGet,
-                                     LogicalFilter, LogicalProject,
-                                     LogicalCreate,
-                                     LogicalCreateUDF, LogicalLoadData,
-                                     LogicalUpload, LogicalQueryDerivedGet,
-                                     LogicalUnion, LogicalOrderBy,
-                                     LogicalLimit, LogicalSample)
-from eva.parser.statement import AbstractStatement
-from eva.parser.select_statement import SelectStatement
-from eva.parser.insert_statement import InsertTableStatement
+from eva.optimizer.operators import (
+    LogicalCreate,
+    LogicalCreateMaterializedView,
+    LogicalCreateUDF,
+    LogicalDrop,
+    LogicalDropUDF,
+    LogicalFilter,
+    LogicalFunctionScan,
+    LogicalGet,
+    LogicalJoin,
+    LogicalLimit,
+    LogicalLoadData,
+    LogicalOrderBy,
+    LogicalProject,
+    LogicalQueryDerivedGet,
+    LogicalRename,
+    LogicalSample,
+    LogicalShow,
+    LogicalUnion,
+    LogicalUpload,
+)
+from eva.optimizer.optimizer_utils import column_definition_to_udf_io
+from eva.parser.create_mat_view_statement import CreateMaterializedViewStatement
 from eva.parser.create_statement import CreateTableStatement
 from eva.parser.create_udf_statement import CreateUDFStatement
-from eva.parser.create_mat_view_statement \
-    import CreateMaterializedViewStatement
+from eva.parser.drop_statement import DropTableStatement
+from eva.parser.drop_udf_statement import DropUDFStatement
+from eva.parser.insert_statement import InsertTableStatement
 from eva.parser.load_statement import LoadDataStatement
-from eva.parser.upload_statement import UploadStatement
-from eva.optimizer.optimizer_utils import column_definition_to_udf_io
+from eva.parser.rename_statement import RenameTableStatement
+from eva.parser.select_statement import SelectStatement
+from eva.parser.show_statement import ShowStatement
+from eva.parser.statement import AbstractStatement
 from eva.parser.table_ref import TableRef
-from eva.utils.logging_manager import LoggingLevel, LoggingManager
+from eva.parser.upload_statement import UploadStatement
+from eva.utils.logging_manager import logger
 
 
 class StatementToPlanConvertor:
@@ -40,23 +57,37 @@ class StatementToPlanConvertor:
         self._dataset = None
 
     def visit_table_ref(self, table_ref: TableRef):
-        """Bind table ref object and convert to Logical get operator
+        """Bind table ref object and convert to LogicalGet, LogicalJoin,
+            LogicalFunctionScan, or LogicalQueryDerivedGet
 
         Arguments:
-            table {TableRef} -- [Input table ref object created by the parser]
+            table {TableRef} - - [Input table ref object created by the parser]
         """
-        if table_ref.is_select():
+        if table_ref.is_table_atom():
+            # Table
+            catalog_vid_metadata = table_ref.table.table_obj
+            self._plan = LogicalGet(table_ref, catalog_vid_metadata, table_ref.alias)
+
+        elif table_ref.is_func_expr():
+            self._plan = LogicalFunctionScan(func_expr=table_ref.func_expr)
+
+        elif table_ref.is_select():
             # NestedQuery
             self.visit_select(table_ref.select_statement)
             child_plan = self._plan
             self._plan = LogicalQueryDerivedGet(table_ref.alias)
             self._plan.append_child(child_plan)
-        else:
-            # Table
-            catalog_vid_metadata = table_ref.table.table_obj
-            self._plan = LogicalGet(table_ref,
-                                    catalog_vid_metadata,
-                                    table_ref.alias)
+
+        elif table_ref.is_join():
+            join_node = table_ref.join_node
+            join_plan = LogicalJoin(
+                join_type=join_node.join_type, join_predicate=join_node.predicate
+            )
+            self.visit_table_ref(join_node.left)
+            join_plan.append_child(self._plan)
+            self.visit_table_ref(join_node.right)
+            join_plan.append_child(self._plan)
+            self._plan = join_plan
 
         if table_ref.sample_freq:
             self._visit_sample(table_ref.sample_freq)
@@ -65,13 +96,12 @@ class StatementToPlanConvertor:
         """converter for select statement
 
         Arguments:
-            statement {SelectStatement} -- [input select statement]
+            statement {SelectStatement} - - [input select statement]
         """
 
         table_ref = statement.from_table
         if table_ref is None:
-            LoggingManager().log('From entry missing in select statement',
-                                 LoggingLevel.ERROR)
+            logger.error("From entry missing in select statement")
             return None
 
         self.visit_table_ref(table_ref)
@@ -84,8 +114,6 @@ class StatementToPlanConvertor:
         # Projection operator
         select_columns = statement.target_list
 
-        # ToDO
-        # add support for SELECT STAR
         if select_columns is not None:
             self._visit_projection(select_columns)
 
@@ -137,9 +165,9 @@ class StatementToPlanConvertor:
         """Converter for parsed insert statement
 
         Arguments:
-            statement {AbstractStatement} -- [input insert statement]
+            statement {AbstractStatement} - - [input insert statement]
+        """
         """
-        '''
         table_ref = statement.table
         table_metainfo = bind_dataset(table_ref.table)
         if table_metainfo is None:
@@ -160,50 +188,74 @@ class StatementToPlanConvertor:
         insert_opr = LogicalInsert(
             table_ref, table_metainfo, statement.column_list, value_list)
         self._plan = insert_opr
-        '''
+        """
 
     def visit_create(self, statement: AbstractStatement):
         """Convertor for parsed insert Statement
 
         Arguments:
-            statement {AbstractStatement} -- [Create statement]
+            statement {AbstractStatement} - - [Create statement]
         """
         table_ref = statement.table_ref
         if table_ref is None:
-            LoggingManager().log("Missing Table Name In Create Statement",
-                                 LoggingLevel.ERROR)
+            logger.error("Missing Table Name In Create Statement")
 
         create_opr = LogicalCreate(
-            table_ref, statement.column_list, statement.if_not_exists)
+            table_ref, statement.column_list, statement.if_not_exists
+        )
         self._plan = create_opr
 
+    def visit_rename(self, statement: RenameTableStatement):
+        """Convertor for parsed rename statement
+        Arguments:
+            statement(RenameTableStatement): [Rename statement]
+        """
+        rename_opr = LogicalRename(statement.old_table_ref, statement.new_table_name)
+        self._plan = rename_opr
+
+    def visit_drop(self, statement: DropTableStatement):
+        drop_opr = LogicalDrop(statement.table_refs, statement.if_exists)
+        self._plan = drop_opr
+
     def visit_create_udf(self, statement: CreateUDFStatement):
         """Convertor for parsed create udf statement
 
         Arguments:
-            statement {CreateUDFStatement} -- Create UDF Statement
+            statement {CreateUDFStatement} - - Create UDF Statement
         """
         annotated_inputs = column_definition_to_udf_io(statement.inputs, True)
-        annotated_outputs = column_definition_to_udf_io(
-            statement.outputs, False)
+        annotated_outputs = column_definition_to_udf_io(statement.outputs, False)
 
-        create_udf_opr = LogicalCreateUDF(statement.name,
-                                          statement.if_not_exists,
-                                          annotated_inputs, annotated_outputs,
-                                          statement.impl_path,
-                                          statement.udf_type)
+        create_udf_opr = LogicalCreateUDF(
+            statement.name,
+            statement.if_not_exists,
+            annotated_inputs,
+            annotated_outputs,
+            statement.impl_path,
+            statement.udf_type,
+        )
         self._plan = create_udf_opr
 
+    def visit_drop_udf(self, statement: DropUDFStatement):
+        """Convertor for parsed DROP UDF statement
+
+        Arguments:
+            statement {DropUDFStatement} - Drop UDF Statement
+        """
+        self._plan = LogicalDropUDF(statement.name, statement.if_exists)
+
     def visit_load_data(self, statement: LoadDataStatement):
         """Convertor for parsed load data statement
         Arguments:
             statement(LoadDataStatement): [Load data statement]
         """
         table_metainfo = statement.table_ref.table.table_obj
-        load_data_opr = LogicalLoadData(table_metainfo,
-                                        statement.path,
-                                        statement.column_list,
-                                        statement.file_options)
+        load_data_opr = LogicalLoadData(
+            table_metainfo,
+            statement.path,
+            statement.column_list,
+            statement.file_options,
+        )
         self._plan = load_data_opr
 
     def visit_upload(self, statement: UploadStatement):
@@ -215,22 +267,26 @@ class StatementToPlanConvertor:
         upload_opr = LogicalUpload(statement.path, statement.video_blob)
         self._plan = upload_opr
 
-    def visit_materialized_view(self,
-                                statement: CreateMaterializedViewStatement):
+    def visit_materialized_view(self, statement: CreateMaterializedViewStatement):
         mat_view_opr = LogicalCreateMaterializedView(
-            statement.view_ref, statement.col_list, statement.if_not_exists)
+            statement.view_ref, statement.col_list, statement.if_not_exists
+        )
 
         self.visit_select(statement.query)
         mat_view_opr.append_child(self._plan)
         self._plan = mat_view_opr
 
+    def visit_show(self, statement: ShowStatement):
+        show_opr = LogicalShow(statement.show_type)
+        self._plan = show_opr
+
     def visit(self, statement: AbstractStatement):
         """Based on the instance of the statement the corresponding
            visit is called.
            The logic is hidden from client.
 
         Arguments:
-            statement {AbstractStatement} -- [Input statement]
+            statement {AbstractStatement} - - [Input statement]
         """
         if isinstance(statement, SelectStatement):
             self.visit_select(statement)
@@ -238,14 +294,22 @@ class StatementToPlanConvertor:
             self.visit_insert(statement)
         elif isinstance(statement, CreateTableStatement):
             self.visit_create(statement)
+        elif isinstance(statement, RenameTableStatement):
+            self.visit_rename(statement)
+        elif isinstance(statement, DropTableStatement):
+            self.visit_drop(statement)
         elif isinstance(statement, CreateUDFStatement):
             self.visit_create_udf(statement)
+        elif isinstance(statement, DropUDFStatement):
+            self.visit_drop_udf(statement)
         elif isinstance(statement, LoadDataStatement):
             self.visit_load_data(statement)
         elif isinstance(statement, UploadStatement):
             self.visit_upload(statement)
         elif isinstance(statement, CreateMaterializedViewStatement):
             self.visit_materialized_view(statement)
+        elif isinstance(statement, ShowStatement):
+            self.visit_show(statement)
         return self._plan
 
     @property
